#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.theme.interfaces import IThemesTarget, IThemesInfo
from pyams_content.shared.view.interfaces import IWfView, IViewQueryEsParamsExtension, IViewThemesSettings
from pyams_content_es.interfaces import IDocumentIndexInfo

# import packages
from elasticsearch_dsl import Q
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.list import unique


@adapter_config(name='themes', context=IThemesTarget, provides=IDocumentIndexInfo)
def ThemesTargetIndexInfo(content):
    """Themes target index info"""
    terms = []
    parents = []
    synonyms = []
    associations = []
    for term in IThemesInfo(content).themes or ():
        terms.append(term.label)
        if term.usage is not None:
            terms.append(term.usage.label)
        parents.extend([parent.label for parent in term.get_parents()])
        synonyms.extend([synonym.label for synonym in term.used_for])
        associations.extend([association.label for association in term.associations])
    return {'themes': {'terms': unique(terms),
                       'parents': unique(parents),
                       'synonyms': unique(synonyms),
                       'associations': unique(associations)}}


@adapter_config(name='themes', context=IWfView, provides=IViewQueryEsParamsExtension)
class ViewThemesQueryEsParamsExtension(ContextAdapter):
    """View themes query adapter """

    weight = 60

    def get_es_params(self, context):
        settings = IViewThemesSettings(self.context)
        themes = settings.get_themes_index(context)
        if themes:
            return Q('terms', **{'themes.terms': themes})
